--
-- This file is part of the ApCoCoA package pool.
--
--   Copyright (c) ApCoCoA Project (Prof. Dr. Martin Kreuzer, Uni Passau)
--
--   Author: 2011 Stefan Schuster
--
-- Visit http://apcocoa.org/ for more information regarding ApCoCoA.
-- Visit http://www.apcocoa.org/wiki/ApCoCoA:KnownIssues for bugs, problems
-- and known issues.
--
-- The ApCoCoA package pool is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License (version 3 or
-- later) as published by the Free Software Foundation. A copy of the full
-- licence may be found in the file COPYING in this directory.
--
-- The ApCoCoA package pool is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with the ApCoCoA package pool; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

-- This file is a template for an apcocoa test package.
-- It demonstrates all features of AUnit and you can derive your own tests
-- from it.
-- For best practices in unit testing we refer you to the jUnit FAQ:
--   http://junit.sourceforge.net/doc/faq/faq.htm


-- To see, how this template works, just enter
--   AUnit.RunTests("elemfns");
-- in ApCoCoA.


-- Bundle your tests related to a particular CoCoA package inside a package 
-- within $apcocoa/ts.
Package $apcocoa/ts/elemfns


Alias Fn   := $apcocoa/elemfns, 
      Asrt := $apcocoa/ts/assert;

-- We do not recommend to define an Initialize() function since it
-- has a special CoCoA semantic. Instead, use OneTimeSetUp(). 


-- The funcion OneTimeSetUp() - if present, it does not have to be -
-- is executed one time before the tests of this test package are executed.
Define OneTimeSetUp()
EndDefine;


-- The funcion OneTimeTearDown() - if present, it does not have to be -
-- is executed one time after the tests of this test package are executed.
Define OneTimeTearDown()
EndDefine;


-- The function SetUp() - if present, it does not have to be - 
-- is executed one time before each test is executed.
Define SetUp()
EndDefine;


-- The function TearDown() - if present, it does not have to be - 
-- is executed one time before each test is executed.
Define TearDown()
EndDefine;


-- The function ExcludeTestsOn() returns a list of pairs (each pair itself
-- a list with two string entries) which are test-platform combinations which
-- are excluded from testing. In particular, you can specify which tests
-- should not be executed on which platform.
--
-- Valid test identifiers are the names of the test functions (without the
-- parenthesis) of this test package or "*" for all test functions.
--
-- The valid platform identifiers are (see also $coclib.GetPlatformString()):
--  * Windows
--  * Linux-x86
--  * Linux-x86_64
--  * MacOSX 
--
Define ExcludeTestsOn()
  -- Exlude all tests on MacOSX and TestTemplate01 on Windows.
  --Return [["TestRound01", "MacOSX"], ["TestPow01", "MacOSX"], ["TestSin", "MacOSX"], 
  --["TestCos", "MacOSX"], ["TestTan", "MacOSX"], ["TestAtan", "MacOSX"],
  --["TestAsin", "MacOSX"], ["TestAcos", "MacOSX"],  ["TestExp", "MacOSX"], ["TestLn", "MacOSX"]];
EndDefine; -- ExcludeTestsOn


Define GetTestAccuracy()
  Return 40;
EndDefine; -- GetTestAccuracy

-- To write tests do the following.
--
-- (1) Define a function for every test you want to realize.
-- (2) Name the functions as follows: TestSomeFeatureOfMyPackage()
--     (you can find examples below or in the other already existing .ts packages)
-- (3) Every function should test only one feature of your package.
-- (4) A test can FAIL or SUCEED. The test fails iff the test functions
--     throws an error. To this end, use the AssertXY functions defined in
--     $apcocoa/ts/assert.cpkg as much as is possible.
--     Note, that your test functions do not return anything.
-- (5) If you miss some AssertXY function, add it to $apcocoa/ts/assert.cpkg.
-- (6) You are free to write an arbitrary number of helper functions inside
--     your test package, just make sure that their 
--     names DO NOT start with 'Test'.
-- (7) To run all the tests defined in the package, call
--       $apcocoa/ts/aunit.RegisterTests("template");
--       $apcocoa/ts/aunit.RunTests();



Define TestRound01()

  TestRing::=QQ[x,y,z];

  Using TestRing Do
    N := 2;
    Fn.SetAccuracy(N);
    Asrt.AssertTrue(Fn.RoundWithAcc(3.131, N) = 3.13,   "Round01");
    Asrt.AssertTrue(Fn.RoundWithAcc(3.135, N) = 3.14,   "Round01");
    Asrt.AssertTrue(Fn.RoundWithAcc(-3.13814234, N) = -3.14, "Round01");
    Asrt.AssertTrue(Fn.RoundWithAcc(-3.134342, N) = -3.13, "Round01");
    Asrt.AssertTrue(Fn.RoundWithAcc(0, N) = 0, "Round01");
    Asrt.AssertTrue(Fn.RoundWithAcc(1423, N) = 1423, "Round01");
    Asrt.AssertTrue(Fn.RoundWithAcc(-1423, N) = -1423, "Round01");
  EndUsing;

EndDefine;


Define TestPow01()
  N := $apcocoa/ts/elemfns.GetTestAccuracy();
  Fn.SetAccuracy(N);
  Asrt.AssertTrue(Fn.Pow(8, 0) = 1, "Pow01");
  Asrt.AssertTrue(Fn.Pow(0, 0) = 1, "Pow01");
  Asrt.AssertTrue(Fn.Pow(2, 1/2) = Fn.Sqrt(2), "Pow01");
  Asrt.AssertTrue(Fn.Pow(2, 1/3) = Fn.Root(2, 3), "Pow01");
  
  Asrt.AssertTrue(Fn.Pow(-8, 1/3) = -2, "Pow: -8^(1/3)");
  
  R := 1.8177087979113040173680779696008422778360594346165952712537119073100516*10^6;
  Asrt.AssertTrue( Fn.Pow(2347, 13/7) = Fn.Round(R), "Pow(2347, 13/7)" );
  
  For J := 1 To 100 Do
    For I := 1 To 100 Do
      Asrt.AssertTrue(Fn.Pow(I, 1/J) = Fn.Root(I, J), "Pow(I, ^/J), I=" + Sprint(I) + ", J=" + Sprint(J));
    EndFor;
  EndFor;
  
EndDefine; -- TestPow01


Define TestSin()
  -- N := 20;
  N := $apcocoa/ts/elemfns.GetTestAccuracy();
  Fn.SetAccuracy(N);
  
  Pi := Fn.PiN(N+1);
  Asrt.AssertTrue(Fn.Sin(0) = 0,    "Sin(0)");
  If N >= 20 Then
    R := 9.99999999999999999999999999999999999833333333333333/10^19;
    Asrt.AssertTrue(Fn.Sin(1/(10^18)) = Fn.RoundWithAcc(R, N),    "Sin(1/10^18)");
  EndIf;
  --
  Asrt.AssertTrue(Fn.Sin(Pi)     = 0,   "Sin(Pi)");
  Asrt.AssertTrue(Fn.Sin(Pi/2)   = 1, "Sin(Pi/2)");
  Asrt.AssertTrue(Fn.Sin(3*Pi/2) = -1, "Sin(3/2Pi)");
  Asrt.AssertTrue(Fn.Sin(Pi/4)   =  Fn.RoundWithAcc(Fn.SqrtN(2, N+1)/2, N), "Sin(Pi/4)");
  Asrt.AssertTrue(Fn.Sin(3*Pi/4) =  Fn.RoundWithAcc(Fn.SqrtN(2, N+1)/2, N), "Sin(3Pi/4)");
  Asrt.AssertTrue(Fn.Sin(5*Pi/4) = -Fn.RoundWithAcc(Fn.SqrtN(2, N+1)/2, N), "Sin(5Pi/4)");
  Asrt.AssertTrue(Fn.Sin(7*Pi/4) = -Fn.RoundWithAcc(Fn.SqrtN(2, N+1)/2, N), "Sin(7Pi/4)");
  Asrt.AssertTrue(Fn.Sin(Pi/3)   = Fn.RoundWithAcc(Fn.SqrtN(3, N+1)/2, N), "Sin(Pi/3)");
  Asrt.AssertTrue(Fn.Sin(Pi/6)   = 1/2, "Sin(Pi/6)");
  Asrt.AssertTrue(Fn.Sin(2*Pi)   = 0,   "Sin(2Pi)");
  --
  Asrt.AssertTrue(Fn.Sin(-Pi) = 0,   "Sin(-Pi)");
  Asrt.AssertTrue(Fn.Sin(-Pi/2) = -1, "Sin(-Pi/2)");
  Asrt.AssertTrue(Fn.Sin(-3*Pi/2) = 1, "Sin(-3/2Pi)");
  Asrt.AssertTrue(Fn.Sin(-Pi/4)   = -Fn.RoundWithAcc(Fn.SqrtN(2, N+1)/2, N), "Sin(-Pi/4)");
  Asrt.AssertTrue(Fn.Sin(-3*Pi/4) = -Fn.RoundWithAcc(Fn.SqrtN(2, N+1)/2, N), "Sin(-3Pi/4)");
  Asrt.AssertTrue(Fn.Sin(-5*Pi/4) = Fn.RoundWithAcc(Fn.SqrtN(2, N+1)/2, N), "Sin(-5Pi/4)");
  Asrt.AssertTrue(Fn.Sin(-7*Pi/4) = Fn.RoundWithAcc(Fn.SqrtN(2, N+1)/2, N), "Sin(-7Pi/4)");
  Asrt.AssertTrue(Fn.Sin(-Pi/3) = -Fn.RoundWithAcc(Fn.SqrtN(3, N+1)/2, N), "Sin(-Pi/3)");
  Asrt.AssertTrue(Fn.Sin(-Pi/6) = -1/2, "Sin(-Pi/6)");
  Asrt.AssertTrue(Fn.Sin(-2*Pi) = 0,   "Sin(-2Pi)");
  -- 
  Asrt.AssertTrue(Fn.Sin(10^100*2*Fn.PiN(N + 102)) = 0, "Sin(2*10^100Pi)");
  Asrt.AssertTrue(Fn.Sin(10^100) =  Fn.RoundWithAcc(-0.372376123661276688262086695553164295719667883567434702364, N), "Sin(10^100)");
  Asrt.AssertTrue(Fn.Sin(1) =       Fn.RoundWithAcc( 0.841470984807896506652502321630298999622563060798371065672, N), "Sin(1)");
  Asrt.AssertTrue(Fn.Sin(-38/377) = Fn.RoundWithAcc(-0.100625165457505252015993914997635916481305830184490156275, N), "Sin(-38/377)");
  
  For I := -10 To 10 Do
    Asrt.AssertTrue(Fn.Sin(I) = Fn.Cos(I-Pi/2), "Sin(I) = Cos(I-Pi/2), I=" + Sprint(I));
    -- Squaring loses accuracy.
    Asrt.AssertTrue(Fn.RoundWithAcc(Fn.Sin(123 + I)^2 + Fn.Cos(123 + I)^2, N-2) = 1, "Sin^2+Cos^2 = 1, I=" + Sprint(I));
    Asrt.AssertTrue(Fn.Sin(I^3) = - Fn.Sin(-I^3), "Sin(I^3) = -Sin(-I^3), I=" + Sprint(I));
  EndFor;
  
EndDefine; -- TestSin


Define TestCos()
  
EndDefine; -- TestCos


Define TestTan()
  N := $apcocoa/ts/elemfns.GetTestAccuracy();
  Fn.SetAccuracy(N);
  
  Pi := Fn.PiN(N+20);
  
  Asrt.AssertTrue(Fn.Tan(0)    = 0, "Tan(0)");
  Asrt.AssertTrue(Fn.Tan(Pi/3) = Fn.Round(Fn.SqrtN(3, N+2)), "Tan(Pi/3)");
  Asrt.AssertTrue(Fn.Tan(Pi/4) = 1, "Tan(Pi/4)");
  Asrt.AssertTrue(Fn.Tan(Pi/6) = Fn.Round(Fn.SqrtN(3, N+2)/3), "Tan(Pi/6)");
  Asrt.AssertTrue(Fn.Tan(Pi/12) = Fn.Round(2-Fn.SqrtN(3, N+2)), "Tan(Pi/12)");
  Asrt.AssertTrue(Fn.Tan(5*Pi/12) = Fn.Round(2+Fn.SqrtN(3, N+2)), "Tan(5Pi/12)");
  Asrt.AssertFail1("$apcocoa/elemfns.Tan", [Pi/2], "Tan(Pi/2)");
  R := Fn.Round(753695.99517414863333585248043853471399301846039504998559876850690056973554497884101206432843601215282);
  Asrt.AssertTrue(Fn.Tan(3.14159/2) = R, "Tan(3.14159/2)");
  R := Fn.Round(-2.6535897932446910952045986398585989007043067560131733030727343284809659520242549215952957754893906195/(10^6));
  Asrt.AssertTrue(Fn.Tan(3.14159) = R, "Tan(3.14159)");
  
  -- These tests are "dangerous" because one has to take into account
  -- the accuracy of pi wrt. the size of the input variable.
  -- E.g., if we let I range in the 10^8's, we have to take pi with a larger
  -- accuracy.
  For I := -10 To 10 Do
    Asrt.AssertTrue(Fn.Tan(-I) = -Fn.Tan(I), "Tan(-I) = -Tan(I), I=" + Sprint(I));
    Asrt.AssertTrue(Fn.Tan(1) = Fn.Tan(1+I*Pi), "Tan(1) = Tan(1+I*Pi), I=" + Sprint(I));
  EndFor;
    
EndDefine; -- TestTan


Define TestAtan()
  N := $apcocoa/ts/elemfns.GetTestAccuracy();
  Fn.SetAccuracy(N);
  
  
  
EndDefine; -- TestAtan


Define TestAsin()
  N := $apcocoa/ts/elemfns.GetTestAccuracy();
  Fn.SetAccuracy(N);
  
  Pi := Fn.PiN(N+2);
  
  R := -Fn.Round(Pi/2);
  Asrt.AssertTrue( Fn.ArcSin(-1)   = R, "ArcSin(-1)");

  R :=Fn.Round(-0.8480620789814810080529443389984180800733662132631126428607181635702008212284742343491898017319572303009952272653);
  Asrt.AssertTrue( Fn.ArcSin(-3/4) = R, "ArcSin(-3/4)");

  R := - Fn.Round(Pi/6);
  Asrt.AssertTrue( Fn.ArcSin(-1/2) = R, "ArcSin(-1/2)");

  R := Fn.Round(-0.2526802551420786534856574369937109722521937330968381936339237787405750604810212224117487422280146016050926029094);
  Asrt.AssertTrue( Fn.ArcSin(-1/4) = R, "ArcSin(-1/4)");

  R := 0;
  Asrt.AssertTrue( Fn.ArcSin(0)    = R, "ArcSin(0)");

  R := Fn.Round(0.2526802551420786534856574369937109722521937330968381936339237787405750604810212224117487422280146016050926029094);
  Asrt.AssertTrue( Fn.ArcSin(1/4)  = R, "ArcSin(1/4)");

  R := Fn.Round(Pi/6);;
  Asrt.AssertTrue( Fn.ArcSin(1/2)  = R, "ArcSin(1/2)");

  R := Fn.Round(0.8480620789814810080529443389984180800733662132631126428607181635702008212284742343491898017319572303009952272653);
  Asrt.AssertTrue( Fn.ArcSin(3/4)  = R, "ArcSin(3/4)");

  R := Fn.Round(Pi/2);
  Asrt.AssertTrue( Fn.ArcSin(1)    = R, "ArcSin(1)");
  
  X := Fn.SqrtN(2, N+2)/2;
  R := Fn.Round(Pi/4);
  Asrt.AssertTrue( Fn.ArcSin(X)    = R, "ArcSin(Sqrt(2)/2)");
  
  X := 1 - 1/(10^(37));
  R := Fn.Round(1.5707963267948966187841080961397935028167499659412976633956218442691034);
  Asrt.AssertTrue( Fn.ArcSin(X) = R, "ArcSin(1-1/(10^37))");
  
  For I := -10 To 10 Do
    Asrt.AssertTrue(Fn.ArcSin(-I/10) = -Fn.ArcSin(I/10), "ArcSin(-I/10) = -ArcSin(I/10), I=" + Sprint(I));
    Asrt.AssertTrue( Fn.Round(Pi/2 - Fn.ArcSinN(I/10, N+2)) = Fn.ArcCos(I/10), "Pi/2-ArcSin(I/10) = ArcCos(I/10)" + Sprint(I) );
    
  EndFor;
  
EndDefine; -- TestAsin


Define TestAcos()
  N := $apcocoa/ts/elemfns.GetTestAccuracy();
  Fn.SetAccuracy(N);
EndDefine; -- TestAcos


Define TestExp()
  N := $apcocoa/ts/elemfns.GetTestAccuracy();
  Fn.SetAccuracy(N);
  
  R := 2135.9497332204142343009568786429406451317837172096487746506973918336541;
  Asrt.AssertTrue( Fn.Exp(23/3) = Fn.Round(R), "Exp(23/3)" );
  
EndDefine; -- TestExp


Define TestLn()
  N := $apcocoa/ts/elemfns.GetTestAccuracy();
  Fn.SetAccuracy(N);
  
  Asrt.AssertTrue(Fn.Ln(1) = 0, "Ln(1)");
  
  R := Fn.Round(0.69314718055994530941723212145817656807550013436025525412068000949339362);
  Asrt.AssertTrue(Fn.Ln(2) = R, "Ln(2)");
  
  For I := -10 To 10 Do
    Asrt.AssertTrue(Fn.Ln(Fn.ExpN(I, 2*N)) = I, "Ln(E^(I)) = I, I=" + Sprint(I));
  EndFor;
  
EndDefine; -- TestLn


Define TestLog()
  N := $apcocoa/ts/elemfns.GetTestAccuracy();
  Fn.SetAccuracy(N);
  
  Asrt.AssertTrue( Fn.Log(10, 1000) = 3, "Log(10, 100) = 3" );
  R := 4.4599651374938994665823406980766208543807541643954502027987311512167999;
  Asrt.AssertTrue( Fn.Log(10, 28838) = Fn.Round(R), "Log(10, 100) = 3" );
  Asrt.AssertTrue( Fn.Log(2, 1024) = 10, "Log(2, 1024) = 3" );
  R := 14.815683492459162527477850372075112689081102620701750505287401189804375;
  Asrt.AssertTrue( Fn.Log(2, 28838) = Fn.Round(R), "Log(2, 28838)" );
  R := -14.815683492459162527477850372075112689081102620701750505287401189804375;
  Asrt.AssertTrue( Fn.Log(1/2, 28838) = Fn.Round(R), "Log(1/2, 28838)" );
  R := -3.4280263732257413619919611274454810325600856335637358295629655807215568;
  Asrt.AssertTrue( Fn.Log(1/20, 28838) = Fn.Round(R), "Log(1/20, 28838)" );
EndDefine; -- TestLog


Define TestSqrt()
  N := $apcocoa/ts/elemfns.GetTestAccuracy();
  Fn.SetAccuracy(N);
  
  For I := 1 To 20 Do
    Asrt.AssertTrue( Fn.Round(Fn.SqrtN(I, N+2)^2) = I, "Sqrt(I)^2 = I, I=" + Sprint(I) );
    Asrt.AssertTrue( Fn.Round(Fn.SqrtN(13/I, N+2)^2) = Fn.Round(13/I), "Sqrt(13/I)^2 = 13/I, I=" + Sprint(I) );
    Asrt.AssertTrue( Fn.Round(Fn.SqrtN(I/217, N+2)^2) = Fn.Round(I/217), "Sqrt(I/217)^2 = I/217, I=" + Sprint(I) );
  EndFor;
  
EndDefine; -- TestSqrt


Define TestNthRoot()
  N := $apcocoa/ts/elemfns.GetTestAccuracy();
  Fn.SetAccuracy(N);
  For J := 1 To 100 Do
    For I := 1 To 100 Do
      Asrt.AssertTrue( Fn.Round(Fn.RootN(I, J, N+4)^(J)) = I , "Root(I, J)^J = I, I=" + Sprint(I) + ", J=" + Sprint(J) );
    EndFor;
  EndFor;
  
EndDefine; -- TestNthRoot

EndPackage;
