--
-- This file is part of the ApCoCoA package pool.
--
--   Copyright (c) ApCoCoA Project (Prof. Dr. Martin Kreuzer, Uni Passau)
--
--   Author: Stefan Schuster
--
-- Visit http://apcocoa.org/ for more information regarding ApCoCoA.
-- Visit http://www.apcocoa.org/wiki/ApCoCoA:KnownIssues for bugs, problems 
-- and known issues.
--
-- The ApCoCoA package pool is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License (version 2 or
-- later) as published by the Free Software Foundation. A copy of the full
-- licence may be found in the file COPYING in this directory.
--
-- The ApCoCoA package pool is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with the ApCoCoA package pool; if not, write to the Free Software
-- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 US
--
Package $apcocoa/ts/string

Alias String := $apcocoa/string,
      A := $apcocoa/ts/assert;


Define TestStringLastIndexOf()
  A.Assert(String.LastIndexOf("abcd", "")     =  5, "TestStringLastIndexOf 1 failed");
  A.Assert(String.LastIndexOf("abcd", "d")    =  4, "TestStringLastIndexOf 2 failed");
  A.Assert(String.LastIndexOf("abcd", "cd")   =  3, "TestStringLastIndexOf 3 failed");
  A.Assert(String.LastIndexOf("abcdcd", "cd") =  5, "TestStringLastIndexOf 4 failed");
  A.Assert(String.LastIndexOf("abcde", "cd")  =  3, "TestStringLastIndexOf 5 failed");
  A.Assert(String.LastIndexOf("abcde", "cdf") = -1, "TestStringLastIndexOf 6 failed");
  A.Assert(String.LastIndexOf("", "")         =  1, "TestStringLastIndexOf 7 failed");
  A.Assert(String.LastIndexOf("", "x")        = -1, "TestStringLastIndexOf 8 failed");
EndDefine;


Define TestStringFirstIndexOf()
  A.AssertTrue( String.FirstIndexOf("abcd", "")     =  0, "TestStringFirstIndexOf 1 failed" );
  A.AssertTrue( String.FirstIndexOf("abcd", "d")    =  4, "TestStringFirstIndexOf 2 failed" );
  A.AssertTrue( String.FirstIndexOf("abcd", "cd")   =  3, "TestStringFirstIndexOf 3 failed" );
  A.AssertTrue( String.FirstIndexOf("abcdcd", "cd") =  3, "TestStringFirstIndexOf 4 failed" );
  A.AssertTrue( String.FirstIndexOf("abcde", "cd")  =  3, "TestStringFirstIndexOf 5 failed" );
  A.AssertTrue( String.FirstIndexOf("abcde", "cdf") = -1, "TestStringFirstIndexOf 6 failed" );
  A.AssertTrue( String.FirstIndexOf("", "")         =  0, "TestStringFirstIndexOf 7 failed" );
  A.AssertTrue( String.FirstIndexOf("", "x")        = -1, "TestStringFirstIndexOf 8 failed" );
EndDefine;


Define TestStringStartsWith()
  A.AssertTrue( String.StartsWith("",    ""),    "TestStringStartsWith 1 failed" );
  A.AssertTrue( String.StartsWith("x",   ""),    "TestStringStartsWith 2 failed" );
  A.AssertTrue( String.StartsWith("abc", "a"),   "TestStringStartsWith 3 failed" );
  A.AssertTrue( String.StartsWith("abc", "ab"),  "TestStringStartsWith 4 failed" );
  A.AssertTrue( String.StartsWith("abc", "abc"), "TestStringStartsWith 5 failed" );
  
  A.AssertFalse( String.StartsWith("abc", "abcd"), "TestStringStartsWith 6 failed" );
  A.AssertFalse( String.StartsWith("abc", "Ab"),   "TestStringStartsWith 7 failed" );
  A.AssertFalse( String.StartsWith("",    "a"),    "TestStringStartsWith 8 failed" );
  A.AssertFalse( String.StartsWith("abc", "bc"),   "TestStringStartsWith 9 failed" );
  A.AssertFalse( String.StartsWith("abc", "xab"),  "TestStringStartsWith 10 failed" );
EndDefine;


Define TestStringEndsWith()
  A.AssertTrue( String.EndsWith("",    ""),    "TestStringEndsWith 1 failed" );
  A.AssertTrue( String.EndsWith("x",   ""),    "TestStringEndsWith 2 failed" );
  A.AssertTrue( String.EndsWith("abc", "c"),   "TestStringEndsWith 3 failed" );
  A.AssertTrue( String.EndsWith("abc", "bc"),  "TestStringEndsWith 4 failed" );
  A.AssertTrue( String.EndsWith("abc", "abc"), "TestStringEndsWith 5 failed" );
  
  A.AssertFalse( String.EndsWith("bcd", "abcd"), "TestStringEndsWith 6 failed" );
  A.AssertFalse( String.EndsWith("abc", "Bc"),   "TestStringEndsWith 7 failed" );
  A.AssertFalse( String.EndsWith("",    "a"),    "TestStringEndsWith 8 failed" );
  A.AssertFalse( String.EndsWith("abc", "ab"),   "TestStringEndsWith 9 failed" );
  A.AssertFalse( String.EndsWith("abc", "bcx"),  "TestStringEndsWith 10 failed" );
EndDefine; -- TestStringEndsWith


Define TestStringFirst()
  A.AssertTrue( String.First("abcde", 0)  = ""     , "TestStringFirst 1 failed" );
  A.AssertTrue( String.First("abcde", 1)  = "a"    , "TestStringFirst 2 failed" );
  A.AssertTrue( String.First("abcde", 2)  = "ab"   , "TestStringFirst 3 failed" );
  A.AssertTrue( String.First("abcde", 5)  = "abcde", "TestStringFirst 4 failed" );
  A.AssertTrue( String.First("abcde", 6)  = "abcde", "TestStringFirst 5 failed" );
  A.AssertTrue( String.First("abcde", -3) = ""     , "TestStringFirst 6 failed" );
EndDefine;


Define TestStringLast()
  A.AssertTrue( String.Last("abcde", 0)  = ""     , "TestStringLast 1 failed" );
  A.AssertTrue( String.Last("abcde", 1)  = "e"    , "TestStringLast 2 failed" );
  A.AssertTrue( String.Last("abcde", 2)  = "de"   , "TestStringLast 3 failed" );
  A.AssertTrue( String.Last("abcde", 5)  = "abcde", "TestStringLast 4 failed" );
  A.AssertTrue( String.Last("abcde", 6)  = "abcde", "TestStringLast 5 failed" );
  A.AssertTrue( String.Last("abcde", -3) = ""     , "TestStringLast 6 failed" );

EndDefine;


-- Define TestSubString()

-- EndDefine;

EndPackage;